<?php
/**
  * Class represents action creates thumbnail (if it doesn't exist) and sends it to user.
  * 
  * @author Marek Blotny <marek@mbsoftware.pl>
  * @copyright Copyright &copy; 2006, Marek Blotny <{@link http://filebrowser.mbsoftware.pl http://filebrowser.mbsoftware.pl}>
  * @license http://www.opensource.org/licenses/mit-license.php The MIT License
  *
  * @package onlinefilebrowser.action
  */ 


/**
  * Class represents action creates thumbnail (if it doesn't exist) and sends it to user.
  * 
  * @author Marek Blotny <marek@mbsoftware.pl>
  * @copyright Copyright &copy; 2006, Marek Blotny <{@link http://filebrowser.mbsoftware.pl http://filebrowser.mbsoftware.pl}>
  * @license http://www.opensource.org/licenses/mit-license.php The MIT License
  *
  * @package onlinefilebrowser.action
  */ 
class GetThumbnailAction extends ActionTemplate {
	
	/**
	  * Creates if it neccessary thumbnail and sends it to the client.
	  * 
	  * @todo remove UploadFormAction :: createThumbnail() call, it's private method.
	  * @access public
	  */
	function perform() {
		
		if ( !isset($_REQUEST['filename']) ) {
			Browser_Utilities :: log("[GetThumbnailAction.perform] lack of file name!", "error");
			return;
		}
		
		$path = stripslashes ($_REQUEST['filename']);
		$filename = basename( $path );
		$relativePath = dirname( $path );
		
		if (!BrowserHelper :: isGraphicalFile($filename)) {
			Browser_Utilities :: log("[GetThumbnailAction.perform] requested file is not a graphical file: ". $filename, "error");
			return;
		}
			
		// base + relative
		$directory = 	Browser :: getBrowserRoot().
				Browser_Utilities :: getSeparator().$relativePath;
		
		// base + relative + file
		$fullpath = 	Browser :: getBrowserRoot().
				Browser_Utilities :: getSeparator().$relativePath.
				Browser_Utilities :: getSeparator().$filename;
				
		if( !file_exists( $fullpath ) ) {
			Browser_Utilities :: log("[GetThumbnailAction.perform] file doesn't exist: ". $fullpath, "error");
			return;
		}
		
		// check if thumbnail exist
		// base + relative + directory + name
		$thumbnail = 	$directory.
				Browser_Utilities :: getSeparator().Browser_Utilities :: getValueFromConfiguration("browser.thumbnail.directory").
				Browser_Utilities :: getSeparator().$filename;
					
		Browser_Utilities :: log("[GetThumbnailAction.perform] file: ". $path, "info");
		Browser_Utilities :: log("[GetThumbnailAction.perform] full path to thumbnail: ". $thumbnail, "info");
		
		$imageToReturn = null;
		
		if ( file_exists( $thumbnail ) ) {
			Browser_Utilities :: log("[GetThumbnailAction.perform] thumbnail exists, it will be created! ", "debug");
			$imageToReturn = ImagesUtils :: createThumbnail($thumbnail);
		} else {
			Browser_Utilities :: log("[GetThumbnailAction.perform] thumbnail doesn't exist! ", "info");
			if ( Browser_Utilities :: getValueFromConfiguration("browser.thumbnail.create") == "true" ) {
				$imageToReturn = ImagesUtils :: createThumbnail($directory . Browser_Utilities :: getSeparator() . $filename);
				// create new thumbnail
				UploadFormAction :: createThumbnail($fullpath);
			} else {
				$imageToReturn = ImagesUtils :: createThumbnail($directory . Browser_Utilities :: getSeparator() . $filename);
			}
		}
		
		
		$mimeType = BrowserHelper :: getMimeType($filename);
		$fileExt = BrowserHelper :: getFileExtension($filename);
		
		header('Content-type: ' . $mimeType );
		
		if ( $fileExt == 'jpeg' || $fileExt == 'jpg') {
			imagejpeg($imageToReturn);
		} else if ($fileExt == 'gif') {
			imagegif($imageToReturn);
		} else if ($fileExt == 'png') {
			imagepng($imageToReturn);
		}
		exit;
	}
}

?>
